# EZRestrict release 7 by David Bowland
# ./addons/eventscripts/ezrestrict/ezrestrict.py

# >>> To configure this addon please see ezrestrict.cfg (created whent he addon is first loaded) <<<

"""
Introduces new server commands to restrict players or teams from weapons and to remove idle weapons. Requires only ES 2.0+
"""


import cfglib
import es
import gamethread
import langlib
import os.path
import playerlib
import random

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'EZRestrict'
info.version  = '7'
info.basename = 'ezrestrict'
info.url      = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author   = 'SuperDave'


###


config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/' + info.basename + '.cfg')

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename':info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')
config.text('>>> THIS ADDON DOES NOTHING UNLESS PROMPTED <<<')
config.text('PLEASE NOTE restrictions only last until the end of the map. Restrictions must be set again for each map.')
config.text('PLEASE ALSO NOTE restrictions can only be set on connected players.')
config.text('\n')

config.text('***** Options *****')
cvar_restrict_repickup     = config.cvar('ezrestrict_restrict_repickup',     2, 'Number of seconds a weapon will be unavaible for pickup after a restricted player attempts to pick the weapon up')
cvar_restrict_removebanned = config.cvar('ezrestrict_restrict_removebanned', 1, '0 = no change, 1 = when a player picks up a weapon both team are restricted from the weapon is removed')
cvar_announce              = config.cvar('ezrestrict_announce',              0, '0 = no change, 1 = players receive a console message when they try to pick up a restricted weapon, 2 = players receive a chat area message when they try to pick up a restricted weapon')
config.text('\n')

config.text('***** Usage documentation *****')
config.text('')
config.text('The new server commands introduced by this addon are:')
config.text('')
config.text('ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')
config.text('')
config.text('ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')
config.text('')
config.text('ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]')
config.text('*** When used without arguments ezrestrict_removeidle removes all idle weapons (excluding the C4)')
config.text('')
config.text('ezrestrict_saferemove <index>')
config.text('*** Ensures the entity exists before removing it (removing a non-existent entity will crash the server)')
config.text('')
config.text('You can use these commands in an addon, with rcon, or directly on the server.')
config.text('\n')
config.text('*** The <player/team> argument accepts: userid / player name / SteamID / #t / @t / #ct / @ct / #all / @all')
config.text('')
config.text('@ arguments apply to teams directly. # arguments apply to members of the team.')
config.text('Players will be prohibited from picking up a weapon if ther weapon is restricted either to the player or the player\'s team.')
config.text('')
config.text('*** The <weapon> argument accepts: weapon name / #all / #primary / #rifle / #shotgun / #smg / #sniper / #secondary / #pistol / #knife / #grenade')
config.text('')
config.text('Below are acceptable weapon names:')
config.text('')
config.text('deagle       - Night Hawk .50C')
config.text('ak47         - CV-47')
config.text('scout        - Schmidt Scout')
config.text('aug          - Bullpup')
config.text('g3sg1        - D3/AU1')
config.text('galil        - IDF Defender')
config.text('famas        - Clarion 5.56')
config.text('m4a1         - Maverick M4A1 Carbine')
config.text('sg552        - Krieg 552')
config.text('sg550        - Krieg 550 Commando')
config.text('m249         - M249')
config.text('awp          - Magnum Sniper Rifle')
config.text('tmp          - Schmidt Machine Pistol')
config.text('mp5navy      - KM Sub-Machine Gun')
config.text('glock        - 9x19mm Sidearm')
config.text('elite        - .40 Dual Elites')
config.text('m3           - Leone 12 Gauge Super')
config.text('xm1014       - Leone YG1265 Auto Shotgun')
config.text('usp          - KM .45 Tactical')
config.text('mac10        - Ingram MAC-10')
config.text('ump45        - KM UMP45')
config.text('p228         - 228 Compact')
config.text('fiveseven    - ES Five-Seven')
config.text('p90          - ES C90')
config.text('hegrenade    - HE Grenade')
config.text('flashbang    - Flashbang')
config.text('smokegrenade - Smoke Grenade')
config.text('knife        - Knife')
config.text('c4           - C4 (Note that C4 is not included in the #all tag)')
config.text('')
config.text('Weapon names can also be preceded with "weapon_"')
config.text('')
config.text('*** The <index> argument accepts only an entity index number.')
config.text('\n')

config.text('***** Event documentation *****')
config.text('')
config.text('This addon creates the following events which can be utilized by other addons similar to standard events:')
config.text('')
config.text('ezrestricted_pickup')
config.text('   event_var(userid)          - Userid of the player who picked up the restricted weapon')
config.text('   event_var(weapon)          - Restricted weapon that was picked up')
config.text('   event_var(longname)        - In-game name of the restricted weapon that was picked up')
config.text('   event_var(weaponindex)     - Index of the restricted weapon that was picked up (returns -1 if the weapon was removed)')
config.text('')
config.text('ezrestrict_player')
config.text('   event_var(userid)          - Userid of the player who was restricted from a weapon')
config.text('   event_var(weapon)          - Weapon that was restricted')
config.text('   event_var(longname)        - In-game name of the weapon that was restricted')
config.text('')
config.text('ezunrestrict_player')
config.text('   event_var(userid)          - Userid of the player who was unrestricted from a weapon')
config.text('   event_var(weapon)          - Weapon that was unrestricted')
config.text('   event_var(longname)        - In-game name of the weapon that was unrestricted')
config.text('')
config.text('Each event also provides event_var(es_steamid), event_var(es_username), and all the other event_vars that accompany')
config.text('event_var(userid). More information can be found here: http://www.eventscripts.com/pages/Extended_event_variables')
config.text('\n')

config.text('***** Python documentation *****')
config.text('')
config.text('In addition to server commands EZRestrict offers Python commands. In order to utilize the Python commands EZRestrict must be loaded')
config.text('before your addon and your addon must contain the import line:')
config.text('')
config.text('from ezrestrict import ezrestrict')
config.text('')
config.text('You may then use the following commands:')
config.text('')
config.text('ezrestrict.restrict(<player or players>, <weapon or weapons>)')
config.text('ezrestrict.unrestrict(<player or players>, <weapon or weapons>)')
config.text('ezrestrict.removeidle([weapon or weapons])')
config.text('ezrestrict.saferemove(<index>)')
config.text('')
config.text('Player arguments can be an integer userid, a string containing a value acceptable as a player argument to the server commands, or any object that can be iterated containing acceptable string or integer values.')
config.text('Weapon arguments can be a string or any object that can be iterated containing values acceptable as weapon arguments to the server commands.')
config.text('Index argument is an integer entity index or any object that can be iterated containing integer entity indexes.')

config.write()


###


def getGameData():
   """Returns the eye angle property based on game"""
   game_data = {'cstrike':('CCSPlayer.m_angEyeAngles[%s]', {'t':2, 'ct':3}, {'deagle':('#all', '#secondary', '#pistol'), 'ak47':('#all', '#primary', '#rifle'), 'scout':('#all', '#primary', '#rifle', '#sniper'), 'aug':('#all', '#primary', '#rifle'), 'g3sg1':('#all', '#primary', '#rifle', '#sniper'), 'galil':('#all', '#primary', '#rifle'), 'famas':('#all', '#primary', '#rifle'), 'm4a1':('#all', '#primary', '#rifle'), 'sg552':('#all', '#primary', '#rifle'), 'sg550':('#all', '#primary', '#rifle', '#sniper'), 'm249':('#all', '#primary'), 'awp':('#all', '#primary', '#rifle', '#sniper'), 'tmp':('#all', '#primary', '#smg'), 'mp5navy':('#all', '#primary', '#smg'), 'glock':('#all', '#secondary', '#pistol'), 'elite':('#all', '#secondary', '#pistol'), 'm3':('#all', '#primary', '#shotgun'), 'xm1014':('#all', '#primary', '#shotgun'), 'usp':('#all', '#secondary', '#pistol'), 'mac10':('#all', '#primary', '#smg'), 'ump45':('#all', '#primary', '#smg'), 'p228':('#all', '#secondary', '#pistol'), 'fiveseven':('#all', '#secondary', '#pistol'), 'p90':('#all', '#primary', '#smg'), 'knife':('#all', '#knife'), 'hegrenade':('#all', '#grenade'), 'flashbang':('#all', '#grenade'), 'smokegrenade':('#all', '#grenade'), 'c4':()}), 'dod':('CDODPlayer.m_angEyeAngles[%s]', {'a':2, 'x':3}, {'30cal':('#all', '#machinegun', '#primary'), 'amerhand':('#all', '#hand'), 'bar':('#all', '#machinegun', '#primary'), 'bazooka':('#all', '#bazooka', '#primary'), 'basebomb':(), 'c96':('#all', '#smg', '#primary'), 'colt':('#all', '#pistol', '#secondary'), 'frag_us':('#all', '#grenade'), 'frag_ger':('#all', '#grenade'), 'garand':('#all', '#rifle', '#primary'), 'k98':('#all', '#rifle', '#primary'), 'k98s':('#all', '#rifle', '#sniper', '#primary'), 'm1carb':('#all', '#rifle', '#primary'), 'mg42':('#all', '#machinegun', '#primary'), 'mp40':('#all', '#smg', '#primary'), 'mp44':('#all', '#smg', '#primary'), 'p38':('#all', '#pistol', '#secondary'), 'punch_ger':('#all', '#grenade'), 'punch_us':('#all', '#grenade'), 'pschreck':('#all', '#bazooka', '#primary'), 'riflegren_ger':('#all', '#grenade'), 'riflegren_us':('#all', '#grenade'), 'smoke_ger':('#all', '#grenade'), 'smoke_us':('#all', '#grenade'), 'spade':('#all', '#hand'), 'spring':('#all', '#rifle', '#sniper', '#primary'), 'stick':('#all', '#grenage'), 'thompson':('#all', '#smg', '#primary')}), 'hl2mp':('CHL2MP_Player.m_angEyeAngles[%s]', {'r':2, 'c':3}, {'357':('#all', '#pistol'), 'alyxgun':('#all', '#pistol'), 'annabelle':('#all', '#shotgun'), 'ar2':('#all', '#smg'), 'brickbat':('#all', '#tool'), 'bugbait':('#all', '#tool'), 'citizenpackage':('#all', '#tool'), 'citizensuitcase':('#all', '#tool'), 'crossbow':('#all', '#sniper'), 'crowbar':('#all', '#hand'), 'extinguisher':('#all', '#tool'), 'frag':('#all', '#grenade'), 'physcannon':('#all', '#tool'), 'physgun':('#all', '#tool'), 'pistol':('#all', '#pistol'), 'rpg':('#all', '#rpg'), 'shotgun':('#all', '#shotgun'), 'slam':('#all', '#hand'), 'smg1':('#all', '#smg'), 'stunstick':('#all', '#hand')})}
   gamename  = str(es.ServerVar('eventscripts_gamedir')).replace('\\', '/').split('/')[~0]

   es.dbgmsg(1, 'EZRestrict: Current game - \"%s\"' % gamename)

   return game_data.get(gamename, game_data['cstrike'])

eyeangle_prop, acceptable_teams, weapon_tags = getGameData()


###


class RestrictedTeam(object):
   teamnum      = 0
   restrictions = set()
   
   def __init__(self, teamnum):
      self.teamnum      = teamnum
      self.restrictions = set()

   def addRestriction(self, weapon):
      """
      Adds the weapon to the set of restricted weapons
      Calls the removeWeapon function on the RestrictedPlayer instance for each player on the restricted team
      """
      weapon = weapon.replace('weapon_', '')
      if weapon in self.restrictions: return

      self.restrictions.add(weapon)

      for userid in filter(lambda x: es.getplayerteam(x) == self.teamnum if not es.getplayerprop(x, 'CBasePlayer.pl.deadflag') else False, es.getUseridList()):
         getPlayer(userid).removeWeapon(weapon)

   def removeRestriction(self, weapon):
      """Removes the weapon from the set of restricted weapons"""
      self.restrictions = self.restrictions.difference((weapon.replace('weapon_', ''),))

   def isRestricted(self, weapon):
      """Returns True if the weapon is restricted otherwise returns False"""
      return weapon in self.restrictions

   def joiningPlayer(self, userid):
      """Removes any restricted weapons from the player joining the team"""
      player = getPlayer(userid)
      if player.isDead(): return

      for weapon in self.restrictions:
         player.removeWeapon(weapon)


restrictedteams = {}


def getTeam(team):
   """Returns a RestrictedTeam instance based on team number"""
   global restrictedteams

   team = int(team)
   if team not in restrictedteams:
      restrictedteams[team] = RestrictedTeam(team)

   return restrictedteams[team]


def clearTeamRestrictions():
   """Clears the RestrictedTeam instances"""
   restrictedteams.clear()


###


class RestrictedPlayer(object):
   userid       = 0
   handle       = 0
   restrictions = set()
   lastgive     = es.ServerVar('eventscripts_lastgive')

   def __init__(self, userid):
      self.userid       = userid
      self.handle       = es.getplayerhandle(userid)
      self.restrictions = set()

   def addRestriction(self, weapon):
      """
      Adds the weapon to the set of restricted weapons
      Fires ezrestrict_player
      """
      weapon = weapon.replace('weapon_', '')
      if weapon in self.restrictions: return

      self.restrictions.add(weapon)

      es.event('initialize', 'ezrestrict_player')
      es.event('setint',     'ezrestrict_player', 'userid',   self.userid)
      es.event('setstring',  'ezrestrict_player', 'weapon',   'weapon_' + weapon)
      es.event('setstring',  'ezrestrict_player', 'longname', lang_text(weapon, {}))
      es.event('fire',       'ezrestrict_player')

      self.removeWeapon(weapon)

   def removeRestriction(self, weapon):
      """
      Removes the weapon from the set of restricted weapons
      Fires ezunrestrict_player
      """
      weapon = weapon.replace('weapon_', '')
      if weapon not in self.restrictions: return

      self.restrictions.remove(weapon)

      es.event('initialize', 'ezunrestrict_player')
      es.event('setint',     'ezunrestrict_player', 'userid',   self.userid)
      es.event('setstring',  'ezunrestrict_player', 'weapon',   'weapon_' + weapon)
      es.event('setstring',  'ezunrestrict_player', 'longname', lang_text(weapon, {}))
      es.event('fire',       'ezunrestrict_player')

   def isRestricted(self, weapon):
      """Returns True if the weapon is restricted otherwise returns False"""
      return weapon in self.restrictions or getTeam(es.getplayerteam(self.userid)).isRestricted(weapon)

   def itemPickup(self, weapon):
      """Calls removeWeapon if the picked up weapon is restricted"""
      weapon = weapon.replace('weapon_', '')

      if self.isRestricted(weapon):
         self.removeWeapon(weapon)
   
   def removeWeapon(self, weapon):
      """
      Executes "lastinv" on the client if the weapon to remove is the player's active weapon
      Removes the weapon by index
      Creates a new weapon at the player's feet if necessary
      Announces the restricted pickup
      """
      longname = 'weapon_' + weapon

      if es.createplayerlist(self.userid)[self.userid]['weapon'] == longname:
         es.cexec(self.userid, 'lastinv')

      for index in es.createentitylist(longname):
         if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') <> self.handle: continue

         gamethread.delayed(0.2, saferemove, index)

         if (getTeam(2).isRestricted(weapon) and getTeam(3).isRestricted(weapon)) if int(cvar_restrict_removebanned) else False:
            lastgive = -1

         else:
            eye_angles = tuple(es.getplayerprop(self.userid, eyeangle_prop % x) for x in range(2))
            es.server.cmd('es_xsetang %s 90 0 0' % self.userid)

            es.server.cmd('es_xentcreate %s %s' % (self.userid, longname))
            lastgive = int(self.lastgive)
            setNPCWeapon(longname)

            es.server.cmd('es_xsetang %s %s %s 0' % ((self.userid,) + eye_angles))

         self.announceRestrictedPickup(weapon, lastgive)

         break

   def announceRestrictedPickup(self, weapon, lastgive=-1):
      """
      Fires ezrestricted_pickup
      Announces to the player the weapon is restricted
      """
      es.event('initialize', 'ezrestricted_pickup')
      es.event('setint',     'ezrestricted_pickup', 'userid',      self.userid)
      es.event('setstring',  'ezrestricted_pickup', 'weapon',      'weapon_' + weapon)
      es.event('setstring',  'ezrestricted_pickup', 'longname',    lang_text(weapon, {}))
      es.event('setint',     'ezrestricted_pickup', 'weaponindex', lastgive)
      es.event('fire',       'ezrestricted_pickup')

      announce = int(cvar_announce)
      if announce == 2:
         es.tell(self.userid, '#multi', lang_text('restricted pick up', {'weapon':weapon}, playerlib.getPlayer(self.userid).get('lang')))
      elif announce:
         es.cexec(self.userid, 'echo ' + self.__removeTags(lang_text('restricted pick up', {'weapon':weapon}, playerlib.getPlayer(self.userid).get('lang'))))

   def isDead(self):
      return es.getplayerprop(self.userid, 'CBasePlayer.pl.deadflag')

   def __removeTags(self, text):
      """Removes #lightgreen, #green and #default tags from text"""
      return text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


restrictedplayers = {}


def getPlayer(userid):
   """Returns a RestrictedPlayer instance based on userid"""
   global restrictedplayers

   userid = int(userid)
   if userid not in restrictedplayers:
      restrictedplayers[userid] = RestrictedPlayer(userid)

   return restrictedplayers[userid]


def removePlayer(userid):
   """Removes RestrictedPlayer instance based on userid"""
   global restrictedplayers

   userid = int(userid)
   if userid in restrictedplayers:
      del restrictedplayers[userid]


def clearPlayers():
   """Clears RestrictedPlayer instances""" 
   restrictedplayers.clear()


###


class NPCWeapon(object):
   name        = ''
   activedelay = False

   def __init__(self, name):
      self.name = name

   def createDelay(self):
      """
      Designates all weapons of the desired type as "reserved for NPC"
      """
      self.removeDelay()

      userid = es.getuserid()
      if userid:
         es.server.cmd('es_xfire %s %s addoutput \"spawnflags 2\"' % (userid, self.name))

         self.activedelay = True
         gamethread.delayedname(cvar_restrict_repickup, 'ezrestrict_weapon_%s' % self.name, self._endDelay)
   
   def removeDelay(self):
      """Removes the delay for this NPC weapon"""
      if self.activedelay:
         gamethread.cancelDelayed('ezrestrict_weapon_%s' % self.name)

         self._endDelay()

   def _endDelay(self):
      """Marks the weapons as available for player pickup"""
      self.activedelay = False

      userid = es.getuserid()
      if userid:
         es.server.cmd('es_xfire %s %s addoutput \"spawnflags 0\"' % (userid, self.name))


npcweapons = {}


def setNPCWeapon(name):
   """Calls createDelay on a NPCWeapon instance based on weapon name"""
   global npcweapons

   if name not in npcweapons:
      npcweapons[name] = NPCWeapon(name)

   npcweapons[name].createDelay()


def clearNPCWeapons():
   """
   Calls removeDelay on each NPCWeapon instance
   Clears all NPCWeapon instances
   """
   for name in npcweapons:
      npcweapons[name].removeDelay()

   npcweapons.clear()


###


def getPlayerList(players):
   """Returns a set of RestrictedPlayer or RestrictedTeam instances based on userid or team tag"""
   if isinstance(players, str):
      if players.startswith('@'):
         if players[1:] == 'all':
            players = (getTeam(2), getTeam(3))
         elif players[1:] in acceptable_teams:
            players = (getTeam(acceptable_teams[players[1:]]),)
         else:
            raise ValueError, 'Invalid team tag \"%s\"' % players

      elif players.startswith('#'):
         if players[1:] == 'all':
            players = es.getUseridList()
         elif players[1:] in acceptable_teams:
            team    = acceptable_teams[players[1:]]
            players = filter(lambda x: es.getplayerteam(x) == team, es.getUseridList())
         else:
            raise ValueError, 'Invalid team tag \"%s\"' % players

         players = map(getPlayer, players)

      else:
         userid = es.getuserid(players)
         if not userid:
            raise ValueError, 'Invalid player \"%s\"' % players

         players = (getPlayer(userid),)

   elif isinstance(players, (int, float)):
      players = (getPlayer(players),)

   return players


def getWeaponList(weapons):
   """Returns a set of weapons based on weapon name or tag"""
   weaponlist = set()
   for weapon in (weapons,) if isinstance(weapons, str) else weapons:
      if weapon.startswith('#'):
         filtered = filter(lambda x: weapon in weapon_tags[x], weapon_tags)
         if not filtered:
            raise ValueError, 'Invalid weapon type \"%s\"' % weapon

         weaponlist = weaponlist.union(filtered)

      elif weapon.replace('weapon_', '') in weapon_tags:
         weaponlist = weaponlist.union((weapon,))

      else:
         raise ValueError, 'Invalid weapon or weapon type \"%s\"' % weapon

   return weaponlist


###


def getLangText():
   lang_path = es.getAddonPath(info.basename) + '/%s_languages.ini' % info.basename
   if os.path.isfile(lang_path):
      return langlib.Strings(lang_path)

   else:
      es.dbgmsg(0, '%(name)s: Unable to load %(basename)s_languages.ini! Please ensure it is in the ./%(basename)s/ directory.' % {'name':info.name, 'basename':info.basename})
      return lambda x, y, z: '%s: %s_languages.ini not found' % (info.name, info.basename)

lang_text = getLangText()


###


def load():
   """
   Checks to make sure critical server variables are created by the config
   """
   config.execute()

   load_events()


def es_map_start(event_var):
   clearTeamRestrictions()
   clearPlayers()
   clearNPCWeapons()

   load_events()


def round_start(event_var):
   clearNPCWeapons()


def dod_round_start(event_var):
   clearNPCWeapons()


def item_pickup(event_var):
   getPlayer(event_var['userid']).itemPickup(event_var['item'])


def player_team(event_var):
   getTeam(event_var['team']).joiningPlayer(event_var['userid'])


def player_disconnect(event_var):
   removePlayer(event_var['userid'])


def unload():
   """Unregisters the client comand filter"""
   clearNPCWeapons()

   es.addons.unregisterClientCommandFilter(_buy_restrict)

   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)


###


def _buy_restrict(userid, args):
   """Client command filter to prevent players from buying restricted weapons"""
   if args[0].lower() == 'buy' and len(args) > 1:
      player = getPlayer(userid)
      weapon = args[1].lower().replace('weapon_', '')
      if player.isRestricted(weapon):
         player.announceRestrictedPickup(weapon)
         return False

   return True
es.addons.registerClientCommandFilter(_buy_restrict)


###


def saferemove_cmd():
   """
   ezrestrict_saferemove <index>
   """
   if es.getargc() == 2:
      saferemove(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: ezrestrict_saferemove <index>')

if not es.exists('command', 'ezrestrict_saferemove'):
   es.regcmd('ezrestrict_saferemove', 'ezrestrict/saferemove_cmd', 'ezrestrict_saferemove <index>\nServer command for safely removing an entity')


def saferemove(arg_index):
   """Ensures the entity at index arg_index exists before it is removed"""
   if int(arg_index) in es.createentitylist(arg_index):
      es.server.cmd('es_xremove %s' % arg_index)


###


def removeidle_cmd():
   """
   ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]
   """
   arg_count = es.getargc()
   if arg_count > 1:
      weapons = getWeaponList([es.getargv(x).lower() for x in range(1, arg_count)])
      if not weapons: raise ValueError, 'Invalid weapon list'

      removeidle(weapons)

   else:
      removeidle()

if not es.exists('command', 'ezrestrict_removeidle'):
   es.regcmd('ezrestrict_removeidle', 'ezrestrict/removeidle_cmd', 'ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]\nServer command for removing idle weapons')


def removeidle(arg_weapons='#all'):
   """Removes idle weapons of the desired type"""
   arg_weapons = getWeaponList(arg_weapons)
   if not arg_weapons: raise ValueError, 'Invalid weapon list'

   for weapon in arg_weapons:
      for index in filter(lambda x: es.getindexprop(x, 'CBaseEntity.m_hOwnerEntity') == -1, es.createentitylist('weapon_' + weapon)):
         es.server.cmd('es_xremove %s' % index)


###


def ezrestrict_cmd():
   """
   ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]
   ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]
   """
   arg_count    = es.getargc()
   command_name = es.getargv(0).lower()
   func_command = restrict if command_name == 'ezrestrict' else unrestrict

   if arg_count > 2:
      func_command(es.getargv(1).lower(), [es.getargv(x).lower() for x in range(2, arg_count)])

   else:
      es.dbgmsg(0, 'Syntax: %s <player/team> <weapon 1> [weapon 2] ... [weapon n]' % command_name)

if not es.exists('command', 'ezrestrict'):
   es.regcmd('ezrestrict', 'ezrestrict/ezrestrict_cmd', 'ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]\nServer command for restricting weapons from players or teams')
if not es.exists('command', 'ezunrestrict'):
   es.regcmd('ezunrestrict', 'ezrestrict/ezrestrict_cmd', 'ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]\nServer command for unrestricting weapons from players or teams')


def restrict(arg_players, arg_weapons):
   """Restricts weapon or weapon tag arg_weapons from player or team arg_players"""
   arg_players = getPlayerList(arg_players)
   if not arg_players: raise ValueError, 'Invalid player list'

   arg_weapons = getWeaponList(arg_weapons)
   if not arg_weapons: raise ValueError, 'Invalid weapon list'

   for item in arg_players:
      for weapon in arg_weapons:
         item.addRestriction(weapon)


def unrestrict(arg_players, arg_weapons):
   """Unrestricts weapon or weapon tag arg_weapons from player or team arg_players"""
   arg_players = getPlayerList(arg_players)
   if not arg_players: raise ValueError, 'Invalid player list'

   arg_weapons = getWeaponList(arg_weapons)
   if not arg_weapons: raise ValueError, 'Invalid weapon list'

   for item in arg_players:
      for weapon in arg_weapons:
         item.removeRestriction(weapon)


###


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('ezrestrict') + '/ezrestrict.res'):
      es.loadevents('declare', 'addons/eventscripts/ezrestrict/ezrestrict.res')

   else:
      events = {'ezrestricted_pickup':{'userid':'short', 'weapon':'string', 'longname':'string', 'weaponindex':'short'}, 'ezrestrict_player':{'userid':'short', 'weapon':'string', 'longname':'string'}, 'ezunrestrict_player':{'userid':'short', 'weapon':'string', 'longname':'string'}}

      if not os.path.isfile(es.getAddonPath('ezrestrict') + 'es_ezrestrict_events_db.txt'):
         es.keygroupcreate('ezrestrict_events')
         for event_name in events:
            es.keycreate('ezrestrict_events', event_name)
            for event_var in events[event_name]:
               es.keysetvalue('ezrestrict_events', event_name, event_var, events[event_name][event_var])
         es.keygroupsave('ezrestrict_events', '|ezrestrict')
         es.keygroupdelete('ezrestrict_events')
      es.loadevents('declare', 'addons/eventscripts/ezrestrict/es_ezrestrict_events_db.txt')
      es.dbgmsg(0, 'EZRestrict: Unable to load ezrestrict.res! Please ensure it is in the ./ezrestrict/ directory.')